# HALtoCHM.awk - Convert a listing of a HAL HELP file to HTML and XML
#  files suitable to create a .CHM file.
#
# The input file contains many sections, each of which looks like:
#  123456789a123456789b...
#  T         CPPTEXT
#   SYSGEN exec file - see PF SINWIDECKS for more information.
#   -------
#  0ENTRY INFORMATION FOR - CPPTEXT
#  0CPPTEXT IS A DATA FILE.
#   HAL WILL COPY CPPTEXT TO LOCAL FILE "PPTEXT".
#
# A "T" in column 1 can appear in the middle of a long entry, so
# it does not necessarily indicate a new topic.
#
# Mark Riordan   16 May 2004

function flush_topic() {
   if("" != file_topic) {
      print "</pre></body></html>" >>file_topic
      close(file_topic)
   }
}

// Not used, after all
function IsAllUpper(str,  j, len, bIsAllUpper) {
   bIsAllUpper = 1
   len = length(str)
   for(j=1; j<=len; j++) {
      if(0==index(substr(str, j, 1), "ABCDEFGHIJKLMNOPQRSTUVWXYZ")) {
         bIsAllUpper =0
         break;
      }
   }
   return bIsAllUpper
}

BEGIN {
   file_toc = "TableOfContents.hhc"
   print "<html><BODY><UL>" >file_toc
   
   file_index = "Index.hhk"
   print "<html><BODY><UL>" >file_index

   while(getline) {
      if("T         " == substr($0, 1, 10) && (2 == NF) && 
       (($2 == toupper($2)) || "Intro" == $2)) {
         flush_topic()
         
         topic = $2
         file_topic = topic ".htm"
         print "<html><head>" >file_topic
         print "<link rel=\"stylesheet\" type=\"text/css\" href=\"mrr.css\">" >>file_topic
         print "<title>" topic "</title>" >>file_topic
         print "</head><body>" >>file_topic
         print "<h2>" topic "</h2><pre>" >>file_topic
         
         print " <LI> <OBJECT type=\"text/sitemap\">" >>file_toc
         print "   <param name=\"Name\" value=\""  topic "\">" >>file_toc
         print "   <param name=\"Local\" value=\"" file_topic "\">" >>file_toc
         print "  </OBJECT>" >>file_toc

         print " <LI> <OBJECT type=\"text/sitemap\">" >>file_index
         print "   <param name=\"Name\" value=\""  topic "\">" >>file_index
         print "   <param name=\"Local\" value=\"" file_topic "\">" >>file_index
         print "  </OBJECT>" >>file_index

      } else if("TDESCRIPTION UNAVAILABLE" == substr($0, 1, 24)) {
         # Ignore this line.
      } else {
         text = substr($0, 2)
         gsub(/\&/, "\\&amp;", text)
         gsub(/\</, "\\&lt;", text)
         gsub(/\>/, "\\&gt;", text)
         print text >>file_topic
      }
   }
   print "</UL></BODY></HTML>" >>file_toc
   print "</UL></BODY></HTML>" >>file_index
}